
/* vsdoc for goog.result */

(function (window) {
    window.goog = window.goog || {};

    window.goog.result = {
        /// <summary></summary>
        /// <returns type="goog.result"/>
                
        wait: function(result, handler, opt_scope) {
            /// <summary>Calls the handler on resolution of the result (success or failure).
            /// The handler is passed the result object as the only parameter. The call will
            /// be immediate if the result is no longer pending.
            /// 
            /// Example:
            /// &lt;pre&gt;
            /// 
            /// var result = xhr.get(&apos;testdata/xhr_test_text.data&apos;);
            /// 
            /// // Wait for the result to be resolved and alert it&apos;s state.
            /// goog.result.wait(result, function(result) {
            ///   alert(&apos;State: &apos; + result.getState());
            /// });
            /// &lt;/pre&gt;</summary>
            /// <param name="result" type="!goog.result.Result">The result to install the handlers.</param>
            /// <param name="handler" type="!function(!goog.result.Result)">The handler to be
            ///     called. The handler is passed the result object as the only parameter.</param>
            /// <param name="opt_scope" type="!Object=">Optional scope for the handler.</param>
        }, 
        
        waitOnSuccess: function(result, handler, opt_scope) {
            /// <summary>Calls the handler if the result succeeds. The result object is the only
            /// parameter passed to the handler. The call will be immediate if the result
            /// has already succeeded.
            /// 
            /// Example:
            /// &lt;pre&gt;
            /// 
            /// var result = xhr.get(&apos;testdata/xhr_test_text.data&apos;);
            /// 
            /// // attach a success handler.
            /// goog.result.waitOnSuccess(result, function(result) {
            ///   var datavalue = result.getvalue();
            ///   alert(&apos;value : &apos; + datavalue);
            /// });
            /// &lt;/pre&gt;</summary>
            /// <param name="result" type="!goog.result.Result">The result to install the handlers.</param>
            /// <param name="handler" type="!function(*|!goog.result.Result)">The handler to be
            ///     called. The handler is passed the result value and the result as
            ///     parameters.</param>
            /// <param name="opt_scope" type="!Object=">Optional scope for the handler.</param>
        }, 
        
        waitOnError: function(result, handler, opt_scope) {
            /// <summary>Calls the handler if the result action errors. The result object is passed as
            /// the only parameter to the handler. The call will be immediate if the result
            /// object has already resolved to an error.
            /// 
            /// Example:
            /// 
            /// &lt;pre&gt;
            /// 
            /// var result = xhr.get(&apos;testdata/xhr_test_text.data&apos;);
            /// 
            /// // Attach a failure handler.
            /// goog.result.waitOnError(result, function(error) {
            ///  // Failed asynchronous call!
            /// });
            /// &lt;/pre&gt;</summary>
            /// <param name="result" type="!goog.result.Result">The result to install the handlers.</param>
            /// <param name="handler" type="!function(!goog.result.Result)">The handler to be
            ///     called. The handler is passed the result object as the only parameter.</param>
            /// <param name="opt_scope" type="!Object=">Optional scope for the handler.</param>
        }, 
        
        transform: function(result, transformer) {
            /// <summary>Given a result and a transform function, returns a new result whose value,
            /// on success, will be the value of the given result after having been passed
            /// through the transform function.
            /// 
            /// If the given result is an error, the returned result is also an error and the
            /// transform will not be called.
            /// 
            /// Example:
            /// &lt;pre&gt;
            /// 
            /// var result = xhr.getJson(&apos;testdata/xhr_test_json.data&apos;);
            /// 
            /// // Transform contents of returned data using &apos;processJson&apos; and create a
            /// // transformed result to use returned JSON.
            /// var transformedResult = goog.result.transform(result, processJson);
            /// 
            /// // Attach success and failure handlers to the tranformed result.
            /// goog.result.waitOnSuccess(transformedResult, function(result) {
            ///   var jsonData = result.getValue();
            ///   assertEquals(&apos;ok&apos;, jsonData[&apos;stat&apos;]);
            /// });
            /// 
            /// goog.result.waitOnError(transformedResult, function(error) {
            ///   // Failed getJson call
            /// });
            /// &lt;/pre&gt;</summary>
            /// <param name="result" type="!goog.result.Result">The result whose value will be
            ///     transformed.</param>
            /// <param name="transformer" type="!Function">The transformer
            ///     function. The return value of this function will become the value of the
            ///     returned result.</param>
            /// <returns type="!goog.result.Result">A new Result whose eventual value will be
            ///     the returned value of the transformer function.</returns>
        }, 
        
        chain: function(result, actionCallback) {
            /// <summary>The chain function aids in chaining of asynchronous Results. This provides a
            /// convenience for use cases where asynchronous operations must happen serially
            /// i.e. subsequent asynchronous operations are dependent on data returned by
            /// prior asynchronous operations.
            /// 
            /// It accepts a result and an action callback as arguments and returns a
            /// result. The action callback is called when the first result succeeds and is
            /// supposed to return a second result. The returned result is resolved when one
            /// of both of the results resolve (depending on their success or failure.) The
            /// state and value of the returned result in the various cases is documented
            /// below:
            /// 
            /// First Result State:    Second Result State:    Returned Result State:
            /// SUCCESS                SUCCESS                 SUCCESS
            /// SUCCESS                ERROR                   ERROR
            /// ERROR                  Not created             ERROR
            /// 
            /// The value of the returned result, in the case both results succeed, is the
            /// value of the second result (the result returned by the action callback.)
            /// 
            /// Example:
            /// &lt;pre&gt;
            /// 
            /// var testDataResult = xhr.get(&apos;testdata/xhr_test_text.data&apos;);
            /// 
            /// // Chain this result to perform another asynchronous operation when this
            /// // Result is resolved.
            /// var chainedResult = goog.result.chain(testDataResult,
            ///     function(testDataResult) {
            /// 
            ///       // The result value of testDataResult is the URL for JSON data.
            ///       var jsonDataUrl = testDataResult.getValue();
            /// 
            ///       // Create a new Result object when the original result is resolved.
            ///       var jsonResult = xhr.getJson(jsonDataUrl);
            /// 
            ///       // Return the newly created Result.
            ///       return jsonResult;
            ///     });
            /// 
            /// // The chained result resolves to success when both results resolve to
            /// // success.
            /// goog.result.waitOnSuccess(chainedResult, function(result) {
            /// 
            ///   // At this point, both results have succeeded and we can use the JSON
            ///   // data returned by the second asynchronous call.
            ///   var jsonData = result.getValue();
            ///   assertEquals(&apos;ok&apos;, jsonData[&apos;stat&apos;]);
            /// });
            /// 
            /// // Attach the error handler to be called when either Result fails.
            /// goog.result.waitOnError(chainedResult, function(result) {
            ///   alert(&apos;chained result failed!&apos;);
            /// });
            /// &lt;/pre&gt;</summary>
            /// <param name="result" type="!goog.result.Result">The result to chain.</param>
            /// <param name="actionCallback" type="!function(!goog.result.Result):!goog.result.Result">The callback called when the result is resolved. This
            ///     callback must return a Result.</param>
            /// <returns type="!goog.result.Result">A result that is resolved when both
            ///     the given Result and the Result returned by the actionCallback have
            ///     resolved.</returns>
        }, 
        
        combine: function(var_args) {
            /// <summary>Returns a result that waits on all given results to resolve. Once all have
            /// resolved, the returned result will succeed (and never error).
            /// 
            /// Example:
            /// &lt;pre&gt;
            /// 
            /// var result1 = xhr.get(&apos;testdata/xhr_test_text.data&apos;);
            /// 
            /// // Get a second independent Result.
            /// var result2 = xhr.getJson(&apos;testdata/xhr_test_json.data&apos;);
            /// 
            /// // Create a Result that resolves when both prior results resolve.
            /// var combinedResult = goog.result.combine(result1, result2);
            /// 
            /// // Process data after resolution of both results.
            /// goog.result.waitOnSuccess(combinedResult, function(results) {
            ///   goog.array.forEach(results, function(result) {
            ///       alert(result.getState());
            ///   });
            /// });
            /// &lt;/pre&gt;</summary>
            /// <param name="var_args" type="...!goog.result.Result">The results to wait on.</param>
            /// <returns type="!goog.result.Result">A new Result whose eventual value will be
            ///     the resolved given Result objects.</returns>
        }, 
        
        combineOnSuccess: function(var_args) {
            /// <summary>Returns a result that waits on all given results to resolve. Once all have
            /// resolved, the returned result will succeed if and only if all given results
            /// succeeded. Otherwise it will error.
            /// 
            /// Example:
            /// &lt;pre&gt;
            /// 
            /// var result1 = xhr.get(&apos;testdata/xhr_test_text.data&apos;);
            /// 
            /// // Get a second independent Result.
            /// var result2 = xhr.getJson(&apos;testdata/xhr_test_json.data&apos;);
            /// 
            /// // Create a Result that resolves when both prior results resolve.
            /// var combinedResult = goog.result.combineOnSuccess(result1, result2);
            /// 
            /// // Process data after successful resolution of both results.
            /// goog.result.waitOnSuccess(combinedResult, function(results) {
            ///   var textData = results[0].getValue();
            ///   var jsonData = results[1].getValue();
            ///   assertEquals(&apos;Just some data.&apos;, textData);
            ///   assertEquals(&apos;ok&apos;, jsonData[&apos;stat&apos;]);
            /// });
            /// 
            /// // Handle errors when either or both results failed.
            /// goog.result.waitOnError(combinedResult, function(combined) {
            ///   var results = combined.getError();
            /// 
            ///   if (results[0].getState() == goog.result.Result.State.ERROR) {
            ///     alert(&apos;result1 failed&apos;);
            ///   }
            /// 
            ///   if (results[1].getState() == goog.result.Result.State.ERROR) {
            ///     alert(&apos;result2 failed&apos;);
            ///   }
            /// });
            /// &lt;/pre&gt;</summary>
            /// <param name="var_args" type="...!goog.result.Result">The results to wait on.</param>
            /// <returns type="!goog.result.Result">A new Result whose eventual value will be
            ///     an array of values of the given Result objects.</returns>
        }, 
        
        Result: function() {
            /// <summary>A Result object represents a value returned by an asynchronous
            /// operation at some point in the future (e.g. a network fetch). This is akin
            /// to a &apos;Promise&apos; or a &apos;Future&apos; in other languages and frameworks.</summary>
        }
        
    };

    var $x = window.goog.result;
    $x.__namespace = "true";
    $x.__typeName = "goog.result";
})(this);
